"""Searches lyrics."""

from .lyrics import LyricsManager
from .models import exceptions  # noqa: F401


def extract_lyrics(url):
    """Try to extract lyrics from url."""
    return LyricsManager.extract_lyrics(url)


def search_lyrics(query, *, google_api_key):
    """Search for lyrics."""
    return LyricsManager.search_lyrics(query, google_api_key)



"""Extractor for lyricsmode.com."""

import logging

from ..extractor import LyricsExtractor
from ..models.lyrics import Lyrics

log = logging.getLogger(__name__)


class Lyricsmode(LyricsExtractor):
    """Class for extracting lyrics."""

    name = "Lyricsmode"
    url = "http://www.lyricsmode.com/"
    display_url = "lyricsmode.com"

    @classmethod
    def extract_lyrics(cls, url_data):
        """Extract lyrics."""
        bs = url_data.bs
        lyrics_window = bs.find_all("p", {"id": "lyrics_text", "class": "ui-annotatable"})[0]
        lyrics = lyrics_window.text

        title = bs.find("h1", attrs={"class": "song_name fs32"}).text.strip()

        return Lyrics(title, lyrics)



"""Extractor for azlyrics.com."""

import logging
import re

from ..extractor import LyricsExtractor
from ..models.lyrics import Lyrics

log = logging.getLogger(__name__)


class AZLyrics(LyricsExtractor):
    """Class for extracting lyrics."""

    name = "AZLyrics"
    url = "https://www.azlyrics.com/"
    display_url = "azlyrics.com"

    @classmethod
    def extract_lyrics(cls, url_data):
        """Extract lyrics."""
        url_data.headers = {"user-agent": "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:40.0) Gecko/20100101 Firefox/40.1"}
        bs = url_data.bs

        center = bs.body.find("div", {"class": "col-xs-12 col-lg-8 text-center"})
        lyrics = center.find("div", {"class": None}).text

        lyrics = re.sub(r"<br>", " ", lyrics)
        lyrics = re.sub(r"<i?>\W*", "[", lyrics)
        lyrics = re.sub(r"\W*<\/i>", "]", lyrics)
        lyrics = re.sub(r"(?=)(\&quot\;)", "\"", lyrics)
        lyrics = re.sub(r"<\/div>", "", lyrics)

        title = center.find("h1").text.strip()[1:-8]

        return Lyrics(title, lyrics)



"""Import all the modules."""
import importlib
import os

imported = []

for module in os.listdir(os.path.dirname(__file__)):
    if module == "__init__.py" or module[-3:] != ".py":
        continue

    name = ".{}".format(module[:-3])
    imported.append(importlib.import_module(name, __name__))

del module



"""Base for extracting."""

import logging

log = logging.getLogger(__name__)


class LyricsExtractorMount(type):
    """Registers new Extractors."""

    def __init__(cls, name, bases, attrs):
        """Add base class to list of extractors."""
        if not hasattr(cls, "extractors"):
            cls.extractors = []
            log.debug("Created Extractor Meta Class")
        else:
            cls.extractors.append(cls)
            log.debug("Registered Extractor \"{}\"".format(name))

    def __str__(self):
        """Return string rep."""
        return "<Extractor {}>".format(self.__name__)


class LyricsExtractor(metaclass=LyricsExtractorMount):
    """A class capable of retrieving lyrics."""

    name = "GENERIC"
    url = "http://giesela.org"
    display_url = "giesela.org"

    @classmethod
    def can_handle(cls, url_data):
        """Check whether this extractor can extract lyrics from this url."""
        return cls.display_url in url_data.url

    @classmethod
    def extract_lyrics(cls, url_data):
        """Return a Lyrics object for the given url, html or bs."""
        raise NotImplementedError



"""Extractor for lyrical-nonsense.com."""

import logging

from ..extractor import LyricsExtractor
from ..models.lyrics import Lyrics

log = logging.getLogger(__name__)


class LyricalNonsense(LyricsExtractor):
    """Class for extracting lyrics."""

    name = "Lyrical Nonsense"
    url = "http://www.lyrical-nonsense.com/"
    display_url = "lyrical-nonsense.com"

    @classmethod
    def extract_lyrics(cls, url_data):
        """Extract lyrics."""
        bs = url_data.bs
        lyrics_window = bs.find_all("div", {"id": "Romaji"})[0] or bs.find_all("div", {"id": "Lyrics"})[0]
        lyrics = lyrics_window.text
        title = bs.select("div.titletext2new h3")[0].text.strip()

        return Lyrics(title, lyrics)



"""Utitlities."""

import re

import requests
from bs4 import BeautifulSoup


class UrlData:
    """Url stuff."""

    def __init__(self, url):
        """Build url."""
        self.url = url
        self.headers = {}

        self.html_parser = "html.parser"

        self._resp = None
        self._html = None
        self._bs = None

    def __str__(self):
        """Return string rep."""
        return "<{}>".format(self.url)

    @property
    def resp(self):
        """Get the requests response object."""
        if not self._resp:
            self._resp = requests.get(self.url, headers=self.headers)
        return self._resp

    @property
    def html(self):
        """Get the html for this url."""
        if not self._html:
            self._html = self.resp.text
        return self._html

    @property
    def bs(self):
        """Get the BeautifulSoup object."""
        if not self._bs:
            self._bs = BeautifulSoup(self.html, self.html_parser)
        return self._bs


def search(query, api_key):
    """Return search results."""
    params = {
        "key": api_key,
        "cx": "002017775112634544492:7y5bpl2sn78",
        "q": query
    }
    resp = requests.get("https://www.googleapis.com/customsearch/v1", params=params)
    data = resp.json()
    items = data.get("items", [])
    return items


def safe_filename(name, file_ending=".json"):
    """Return a safe version of name + file_type."""
    filename = re.sub(r"\s+", "_", name)
    filename = re.sub(r"\W+", "-", filename)

    return filename.lower().strip() + file_ending
